/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "credits.h"

#include "ppgltk/audio/audio.h"

#include "game_config.h"
#include "fonts.h"
#include "gl_util.h"
#include "render_util.h"

#include "ppgltk/ui_mgr.h"
#include "ppgltk/ui_snow.h"
#include "ppgltk/ui_theme.h"


#define CREDITS_MAX_Y -140
#define CREDITS_MIN_Y 64

typedef struct {
    char *binding;
    char *text;
} credit_line_t;

static credit_line_t credit_lines[] = 
{
	{ "credits_h1", "PlanetPenguin Racer" },
    { "credits_text", "http://racer.planetpenguin.de" },
	{ "credits_text", "PlanetPenguin Racer is based on Tux Racer" },
    { "credits_text", "Tux Racer is a Sunspire Studios Production" },
    { "credits_text", "www.sunspirestudios.com" },
    { "credits_text", "" },
    { "credits_h2", "PlanetPenguin Racer Development Team" },
    { "credits_text", "Volker Stroebel" },
	{ "credits_text", "Peter Reichel" },
    { "credits_text", "" },
    { "credits_h2", "Tux Racer Development Team" },
    { "credits_text_small", "(Alphabetical Order)" },
    { "credits_text", "Patrick \"Pog\" Gilhuly" },
    { "credits_text", "Eric \"Monster\" Hall" },
    { "credits_text", "Rick Knowles" },
    { "credits_text", "Vincent Ma" },
    { "credits_text", "Jasmin Patry" },
    { "credits_text", "Mark Riddell" },
    { "credits_text", "" },
    { "credits_h2", "Music" },
    { "credits_text", "Joseph Toscano" },
    { "credits_text", "" },
    { "credits_h2", "Thanks To:" },
    { "credits_text_small", "(In No Particular Order)" },
    { "credits_text", "Larry Ewing" },
    { "credits_text", "Thatcher Ulrich" },
    { "credits_text", "Steve Baker" },
    { "credits_text", "Ingo Ruhnke" },
    { "credits_text", "James Barnard" },
    { "credits_text", "Alan Levy" },
    { "credits_text", "University of Waterloo Computer Graphics Lab" },
    { "credits_text", "" },
    { "credits_text_small", "Tux Racer is a trademark of Jasmin F. Patry" },
	{ "credits_text_small", "PP Racer is copyright 2003, 2004 The PPRacer team" },
    { "credits_text_small", "Tux Racer is copyright 1999, 2000, 2001 Jasmin F. Patry" },
};

Credits::Credits()
{
	winsys_set_mouse_func( loop_mouse_func );
    yOffset = 0;
    play_music( "credits_screen" );
}


Credits::~Credits()
{	
}

void
Credits::loop(double timeStep)
{
	    int width, height;
    width = getparam_x_resolution();
    height = getparam_y_resolution();

    update_audio();

    clear_rendering_context();

    set_gl_options( GUI );

    UIMgr.setupDisplay();

    drawText( timeStep );

    if (getparam_ui_snow()) {
		update_ui_snow( timeStep, false );
		draw_ui_snow();
    }

	theme.drawMenuDecorations();

    UIMgr.draw();

    reshape( width, height );

    winsys_swap_buffers();
}

void
Credits::drawText( double timeStep )
{
    int w = getparam_x_resolution();
    int h = getparam_y_resolution();
    font_t font;
    unsigned int i;
    double y;
    int string_w, asc, desc;

    yOffset += timeStep * 30;
    y = CREDITS_MIN_Y+yOffset;

    glPushMatrix();
    {
	glTranslatef( w/2, y, 0 );

	for (i=0; i<sizeof( credit_lines ) / sizeof( credit_lines[0] ); i++) {
	    credit_line_t line = credit_lines[i];

	    if ( !get_font_binding( line.binding, font ) ) {
		print_warning( IMPORTANT_WARNING,
			       "Couldn't get font for binding %s", 
			       line.binding );
	    } else {
		get_font_metrics( &font, line.text, &string_w, &asc, &desc );

		glTranslatef( 0, -asc, 0 );
		y += -asc;

		glPushMatrix();
		{
		    bind_font_texture( &font );
		    glTranslatef( -string_w/2, 0, 0 );
		    draw_string( &font, line.text );
		}
		glPopMatrix();

		glTranslatef( 0, -desc, 0 );
		y += -desc;
	    }
	}

    }
    glPopMatrix();

    if ( y > h+CREDITS_MAX_Y ) {
	yOffset = 0;
    }

    // Draw strips at the top and bottom to clip out text 
    glDisable( GL_TEXTURE_2D );

    glColor4dv( (double*)&theme.background );

    glRectf( 0, 0, w, CREDITS_MIN_Y );

    glBegin( GL_QUADS );
    {
	glVertex2f( 0, CREDITS_MIN_Y );
	glVertex2f( w, CREDITS_MIN_Y );
	glColor4f( theme.background.r, 
		   theme.background.g,
		   theme.background.b,
		   0 );
	glVertex2f( w, CREDITS_MIN_Y + 30 );
	glVertex2f( 0, CREDITS_MIN_Y + 30 );
    }
    glEnd();

    glColor4dv( (double*)&theme.background );

    glRectf( 0, h+CREDITS_MAX_Y, w, h );

    glBegin( GL_QUADS );
    {
	glVertex2f( w, h+CREDITS_MAX_Y );
	glVertex2f( 0, h+CREDITS_MAX_Y );
	glColor4f( theme.background.r, 
		   theme.background.g,
		   theme.background.b,
		   0 );
	glVertex2f( 0, h+CREDITS_MAX_Y - 30 );
	glVertex2f( w, h+CREDITS_MAX_Y - 30 );
    }
    glEnd();

    glColor4f( 1, 1, 1, 1 );

    glEnable( GL_TEXTURE_2D );
}

bool
Credits::mouseButtonReleaseEvent(int button, int x, int y)
{
	set_game_mode( GAME_TYPE_SELECT );
    winsys_post_redisplay();
	return true;
}

bool
Credits::keyReleaseEvent(SDLKey key)
{
	set_game_mode( GAME_TYPE_SELECT );
    winsys_post_redisplay();
	return true;
}
