/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "game_over.h"


#include "ppgltk/ui_mgr.h"

#include "ppgltk/audio/audio.h"

#include "game_config.h"
#include "gl_util.h"
#include "fps.h"
#include "render_util.h"
#include "phys_sim.h"
#include "view.h"
#include "course_render.h"
#include "tux.h"
#include "tux_shadow.h"
#include "loop.h"
#include "fog.h"
#include "viewfrustum.h"
#include "fonts.h"
#include "hud.h"
#include "joystick.h"
#include "part_sys.h"

#include "stuff.h"
#include "game_mgr.h"

GameOver::GameOver()
{
	halt_sound( "flying_sound" );
    halt_sound( "rock_sound" );
    halt_sound( "ice_sound" );
    halt_sound( "snow_sound" );

    play_music( "game_over" );

    aborted = gameMgr->wasRaceAborted();

	if ( !aborted ) {
		gameMgr->updatePlayersScores();
    }
		
    if ( gameMgr->gametype!=GameMgr::PRACTICING ) {
		bestScore = gameMgr->updateCurrentRaceData();		
		
		if(!gameMgr->wasRaceWon()){
			players[0].decLives();
		}
    }else{
		if ( !aborted ) {
			bestScore = players[0].updateOpenCourseData(
									gameMgr->getCurrentRace().name,
									gameMgr->time,
									players[0].herring,
									players[0].score);
			
			if(bestScore) players[0].saveData();
		}
	}	
}

GameOver::~GameOver()
{
}

void
GameOver::loop(double timeStep)
{
    int width, height;
    width = getparam_x_resolution();
    height = getparam_y_resolution();

    /* Check joystick */
    if ( is_joystick_active() ) {
	update_joystick();

	if ( is_joystick_continue_button_down() )
	{
	    if ( gameMgr->gametype != GameMgr::PRACTICING ) {
			set_game_mode( EVENT_RACE_SELECT );
		}else{
			set_game_mode( RACE_SELECT );
		}
	    winsys_post_redisplay();
	    return;
	}
    }
    fpsCounter.update();

    update_audio();

    clear_rendering_context();

    fogPlane.setup();
    update_player_pos( players[0], 0 );
    update_view( players[0], 0 );

    setup_view_frustum( players[0], NEAR_CLIP_DIST, 
			getparam_forward_clip_distance() );

    draw_sky(players[0].view.pos);
    draw_fog_plane();

    set_course_clipping( true );
    set_course_eye_point( players[0].view.pos );
    setup_course_lighting();
    render_course();
    draw_trees();
    if ( getparam_draw_particles() ) {
		draw_particles( players[0] );
    }

    draw_tux();
    draw_tux_shadow();
    set_gl_options( GUI );

    UIMgr.setupDisplay();
    drawText();

    HUD1.draw(players[0]);
    reshape( width, height );

    winsys_swap_buffers();
}

bool
GameOver::mouseButtonEvent(int button, int x, int y, bool pressed)

{
	if ( gameMgr->gametype!=GameMgr::PRACTICING ) {
		set_game_mode( EVENT_RACE_SELECT );
	}else{
		set_game_mode( RACE_SELECT );
	}
	winsys_post_redisplay();
	return true;
}

bool
GameOver::keyPressEvent(SDLKey key)
{
	if ( gameMgr->gametype!=GameMgr::PRACTICING ) {
		set_game_mode( EVENT_RACE_SELECT );
	}else{
		set_game_mode( RACE_SELECT );
	}
	winsys_post_redisplay();
	return true;
}

void
GameOver::drawText()
{
    int w = getparam_x_resolution();
    int h = getparam_y_resolution();
    int x_org, y_org;
    int box_width, box_height;
    char *string;
    int string_w, asc, desc;
    char buff[BUFF_LEN];
    font_t font;
    font_t stat_label_font;

    box_width = 200;
    box_height = 250;

    x_org = int(w/2.0 - box_width/2.0);
    y_org = int(h/2.0 - box_height/2.0);

    if ( !get_font_binding( "race_over", font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding race_over" );
    } else {
	string = "Race Over";
	get_font_metrics( &font, string, &string_w, &asc, &desc );

	glPushMatrix();
	{
	    glTranslatef( x_org + box_width/2.0 - string_w/2.0,
			  y_org + box_height - asc, 
			  0 );
	    bind_font_texture( &font );
	    draw_string( &font, string );
	}
	glPopMatrix();
    }

    /* If race was aborted, don't print stats */
    if ( !gameMgr->wasRaceAborted() ) {
	if ( !get_font_binding( "race_stats_label", stat_label_font ) ||
	     !get_font_binding( "race_stats", font ) )
	{
	    print_warning( IMPORTANT_WARNING,
			   "Couldn't get fonts for race stats" );
	} else {
	    int asc2;
	    int desc2;
	    get_font_metrics( &font, "", &string_w, &asc, &desc );
	    get_font_metrics( &stat_label_font, "", &string_w, &asc2, &desc2 );
	    if ( asc < asc2 ) {
		asc = asc2;
	    }
	    if ( desc < desc2 ) {
		desc = desc2;
	    }

	    glPushMatrix();
	    {
		int minutes;
		int seconds;
		int hundredths;

		glTranslatef( x_org,
			      y_org + 150,
			      0 );

		bind_font_texture( &stat_label_font );
		draw_string( &stat_label_font, "Time: " );
		getTimeComponents( gameMgr->time, minutes, seconds, hundredths );

		sprintf( buff, "%02d:%02d.%02d", minutes, seconds, hundredths );
		bind_font_texture( &font );
		draw_string( &font, buff );
	    }
		glPopMatrix();

	    glPushMatrix();
	    {
		glTranslatef( x_org,
			      y_org + 150 - (asc + desc),
			      0 );

		bind_font_texture( &stat_label_font );
		draw_string( &stat_label_font, "Herring: " );

		sprintf( buff, "%3d", players[0].herring );

		bind_font_texture( &font );
		draw_string( &font, buff );
	    }
	    glPopMatrix();

	    glPushMatrix();
	    {
		glTranslatef( x_org,
			      y_org + 150 - 2*(asc + desc),
			      0 );
		bind_font_texture( &stat_label_font );
		draw_string( &stat_label_font, "Score: " );

		sprintf( buff, "%6d", players[0].score );

		bind_font_texture( &font );
		draw_string( &font, buff );
	    }
	    glPopMatrix();
			
		glPushMatrix();
	    {
		int speed;
		glTranslatef( x_org,
			      y_org + 150 - 3*(asc + desc),
			      0 );

		bind_font_texture( &stat_label_font );
		draw_string( &stat_label_font, "Max speed: " );

		speed = int((double)players[0].max_speed * M_PER_SEC_TO_KM_PER_H);
			
		sprintf( buff, "%3d km/h", speed);

		bind_font_texture( &font );
		draw_string( &font, buff );
	    }
	    glPopMatrix();
	}
    }

	if ( gameMgr->wasRaceAborted() ) {
		string = "Race aborted.";
    } else if ( gameMgr->gametype==GameMgr::PRACTICING){
		if(bestScore){
			string = "You beat your best score!";
		}else{
			string = "";
		}
	} else if(gameMgr->wasEventWon()){
		string = "Congratulations! You won the event!";
	} else if(gameMgr->wasCupWon()){
		string = "Congratulations! You won the cup!";
	} else if(gameMgr->wasRaceWon()){
		string = "You advanced to the next race!";
	} else {
		string = "You didn't advance.";
    }
	
    if ( !get_font_binding( "race_result_msg", font ) ) {
	print_warning( IMPORTANT_WARNING, 
		       "Couldn't get font for binding race_result_msg" );
    } else {
	get_font_metrics( &font, string, &string_w, &asc, &desc );
	glPushMatrix();
	{
	    glTranslatef( x_org + box_width/2. - string_w/2.,
			  y_org + desc,
			  0 );
	    bind_font_texture( &font );
	    draw_string( &font, string );
	}
	glPopMatrix();
    }
}
