/* 
 * PPRacer 
 * Copyright (C) 2004 Volker Stroebel <mmv1@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "keyboardconfig.h"

#include "fonts.h"
#include "game_config.h"

#include "game_mgr.h"

#include "ppgltk/ui_mgr.h"
	
struct KeyMap{
	SDLKey key;
	char string[10];
};

static KeyMap keymap[] = {
	{SDLK_LEFT,"left"},
	{SDLK_RIGHT,"right"},
	{SDLK_UP,"up"},
	{SDLK_DOWN,"down"},
	{SDLK_KP0,"keypad 0"},
	{SDLK_KP1,"keypad 1"},
	{SDLK_KP2,"keypad 2"},
	{SDLK_KP3,"keypad 3"},
	{SDLK_KP4,"keypad 4"},
	{SDLK_KP5,"keypad 5"},
	{SDLK_KP6,"keypad 6"},
	{SDLK_KP7,"keypad 7"},
	{SDLK_KP8,"keypad 8"},
	{SDLK_KP9,"keypad 9"},
	{SDLK_KP_PERIOD,"keypad ."},
	{SDLK_KP_DIVIDE,"keypad /"},
	{SDLK_KP_MULTIPLY,"keypad *"},
	{SDLK_KP_MINUS,"keypad m"},
	{SDLK_KP_PLUS,"keypad +"},
	{SDLK_KP_EQUALS,"keypad ="}	
};

	
	
KeyboardConfig::KeyboardConfig()
{
	title="Keyboard Configuration";
	
	pp::Vec2d pos(0,0);
	pp::Vec2d size(150,32);
	
	leftEntry = new pp::Entry(pos, size,
				     "listbox_item");
	leftEntry->setEditable(false);
	leftEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	leftEntry->setContent(getKey(SDLKey(getparam_turn_left_key())));
	
	rightEntry = new pp::Entry(pos, size,
				     "listbox_item");
	rightEntry->setEditable(false);
	rightEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	rightEntry->setContent(getKey(SDLKey(getparam_turn_right_key())));
	
	paddleEntry = new pp::Entry(pos, size,
				     "listbox_item");
	paddleEntry->setEditable(false);
	paddleEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	paddleEntry->setContent(getKey(SDLKey(getparam_paddle_key())));
	
	brakeEntry = new pp::Entry(pos, size,
				     "listbox_item");
	brakeEntry->setEditable(false);
	brakeEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	brakeEntry->setContent(getKey(SDLKey(getparam_brake_key())));
	
	jumpEntry = new pp::Entry(pos, size,
				     "listbox_item");
	jumpEntry->setEditable(false);
	jumpEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	jumpEntry->setContent(getKey(SDLKey(getparam_jump_key())));
	
	trickEntry = new pp::Entry(pos, size,
				     "listbox_item");
	trickEntry->setEditable(false);
	trickEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	trickEntry->setContent(getKey(SDLKey(getparam_trick_modifier_key())));
	
	resetEntry = new pp::Entry(pos, size,
				     "listbox_item");
	resetEntry->setEditable(false);
	resetEntry->signalKeyPressed.Connect(pp::CreateSlot(this,&KeyboardConfig::setKey));
   	resetEntry->setContent(getKey(SDLKey(getparam_reset_key())));	
}

KeyboardConfig::~KeyboardConfig()
{
	delete leftEntry;
	delete rightEntry;
	delete paddleEntry;
	delete brakeEntry;
	delete jumpEntry;
	delete trickEntry;
	delete resetEntry;
}

void
KeyboardConfig::setWidgetPositions()
{
	int width = 500;
	int height = 240;
		
	pp::Vec2d pos(getparam_x_resolution()/2 - width/2,
				  getparam_y_resolution()/2 + height/2);
	
	Font font("button_label");
	
	font.draw("Turn left:",pos);
	leftEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
		
	pos.y-=40;
	font.draw("Turn right:",pos);
	rightEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
	
	pos.y-=40;
	font.draw("Paddle:",pos);
	paddleEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
	
	pos.y-=40;
	font.draw("Brake:",pos);
	brakeEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
	
	pos.y-=40;
	font.draw("Jump:",pos);
	jumpEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
	
	pos.y-=40;
	font.draw("Trick:",pos);
	trickEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));

	pos.y-=40;
	font.draw("Reset:",pos);
	resetEntry->setPosition(pp::Vec2d(pos.x+width-150,pos.y));
}

void
KeyboardConfig::apply()
{
	setparam_turn_left_key(getKey(leftEntry->getContent()));
	setparam_turn_right_key(getKey(rightEntry->getContent()));		
	setparam_paddle_key(getKey(paddleEntry->getContent()));		
	setparam_brake_key(getKey(brakeEntry->getContent()));		
	setparam_jump_key(getKey(jumpEntry->getContent()));		
	setparam_trick_modifier_key(getKey(trickEntry->getContent()));		
	setparam_reset_key(getKey(resetEntry->getContent()));		
	
	write_config_file();
	set_game_mode( GameMode::prevmode );
    UIMgr.setDirty();
}

void
KeyboardConfig::setKey(pp::Entry* widget, SDLKey key)
{
	std::string content="";
	
	if(key==SDLK_BACKSPACE){
		widget->setContent(content);
		return;
	}else{
		content = getKey(key);
	}
	
	if (content!=""){
		widget->setContent(content);
	}
}

std::string KeyboardConfig::getKey(SDLKey key)
{
	std::string content="";
	if(isalnum(key)){
		content=key;
	}else{
		for(unsigned int i=0;i<sizeof(keymap)/sizeof(KeyMap);i++){
			if(key==keymap[i].key){
				content=keymap[i].string;
				break;
			}
		}
	}
	return content;
}

SDLKey
KeyboardConfig::getKey(std::string& string)
{
	if(string!=""){
		for(unsigned int i=0;i<sizeof(keymap)/sizeof(KeyMap);i++){
			if(string==keymap[i].string){
				return keymap[i].key;
			}
		}
		return SDLKey(string.c_str()[0]);		
	}else{
		return SDLKey(0);
	}
}
