/* 
 * PPRacer 
 * Copyright (C) 2004 Volker Stroebel <mmv1@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "entry.h"

#include "ui_theme.h"
#include "render_util.h"
#include "fonts.h"

#include "alg/glhelper.h"


namespace pp {
		
Entry::Entry( pp::Vec2d pos, pp::Vec2d size,
			 char *binding, char *content )
{
	this->position = pos;
	this->size = size;
    this->binding = binding;
	this->content = content;
	
	editable=true;
	maxchars=8;
	
	setEnabled(true);
}

void
Entry::draw()
{
	if(hasFocus){
		ppGL::draw::rect(theme.highlight,
				position,
				position+size);	
	}else{	
		ppGL::draw::rect(theme.foreground,
				position,
				position+size);
	}
	ppGL::draw::rect(theme.background,
				position+theme.border,
				position+size-theme.border);
	
	Font font(binding);
	font.draw( content, position+theme.textPadding );	
}

void
Entry::performClickAction()
{
	setFocus(true);
}

void
Entry::setContent(std::string string)
{
	content = string;
}

std::string&
Entry::getContent()
{
	return content;
}



bool
Entry::keyboardEvent(SDLKey key, SDLMod mod, bool release)
{
	if(!release){
		if(editable){
			if( islower(key) ){
				if(content.size()<maxchars){
					if(mod & KMOD_SHIFT){
						content+=toupper(key);
					}else{
						content+=key;
					}
				}
			}else if( isdigit(key) ){
				if(content.size()<maxchars){
					content+=key;
				}
			}else if( key == SDLK_BACKSPACE){
				if(!content.empty()){			
					content.erase(--content.end());
				}
			}
		}else{
			signalKeyPressed.Emit(this,key);
		}
	}
	return true;
}

} //namespace pp
