/* 
 * PPRacer 
 * Copyright (C) 2003-2004 The PPRacer Team
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "race_select.h"


#include "game_config.h"

#include "ppgltk/ui_mgr.h"
#include "ppgltk/ui_theme.h"
#include "ppgltk/button.h"
#include "ppgltk/ssbutton.h"
#include "ppgltk/listbox.h"
#include "ppgltk/ui_snow.h"
#include "ppgltk/textarea.h"

#include "ppgltk/audio/audio.h"

#include "loop.h"
#include "render_util.h"
#include "gl_util.h"
#include "course_load.h"
#include "fonts.h"
#include "textures.h"
#include "joystick.h"

#include "stuff.h"
#include "game_mgr.h"

#include <iostream>



static std::list<CourseData> *courseList;
static std::list<CourseData>::iterator curElem;
static std::list<CourseData>::iterator lastCompletedRace;

RaceSelect::RaceSelect()
{
    pp::Vec2d pos(0,0);

	courseList = &openCourseList;
		
    // Unless we're coming back from a race, initialize the race data to 
    //   defaults.
    
    if ( GameMode::prevmode != GAME_OVER ) {
		// Initialize the race data 
		curElem = courseList->begin();
		gameMgr->setCurrentRace(curElem);
		
	    gameMgr->getCurrentRace().mirrored = false;
	    gameMgr->getCurrentRace().conditions = RACE_CONDITIONS_SUNNY;
	    gameMgr->getCurrentRace().windy = false;
	    gameMgr->getCurrentRace().snowing = false;
    }

    back_btn = new pp::Button(pos,
			      pp::Vec2d(150, 40), 
			      "button_label", 
			      "Back" );
    back_btn->setHilitFontBinding( "button_label_hilit" );
    back_btn->signalClicked.Connect(pp::CreateSlot(this,&RaceSelect::back));

	
    start_btn = new pp::Button(pos,
			       pp::Vec2d(150, 40),
			       "button_label",
			       "Race!" );
    start_btn->setHilitFontBinding( "button_label_hilit" );
    start_btn->setDisabledFontBinding( "button_label_disabled" );
	start_btn->signalClicked.Connect(pp::CreateSlot(this,&RaceSelect::start));

  
	race_listbox = new pp::Listbox<CourseData>(pos,
				   pp::Vec2d(460, 44),
				   "listbox_item",
				   *courseList);
    race_listbox->setCurrentItem( curElem );
	race_listbox->signalChange.Connect(pp::CreateSlot(this,&RaceSelect::listboxItemChange));
    // Create text area 
     
    desc_ta = new pp::Textarea( pos,
			       pp::Vec2d(312, 107),
			       "race_description",
			       "" );
	desc_ta->setText( (*curElem).description );
    // Create state buttons - only if practicing or if cup_complete
    
	// mirror 
	mirror_ssbtn = new pp::SSButton( pos,
					pp::Vec2d(32, 32),
					2 );
	mirror_ssbtn->setStateImage(0,"mirror_button",
				  pp::Vec2d( 0.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 32.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	mirror_ssbtn->setStateImage(1,"mirror_button",
				  pp::Vec2d( 32.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 64.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	mirror_ssbtn->setState( (int)gameMgr->getCurrentRace().mirrored );
	// conditions
	conditions_ssbtn = new pp::SSButton( pos,
					    pp::Vec2d(32, 32),
					    4 );
	conditions_ssbtn->setStateImage(0,"conditions_button",
				  pp::Vec2d( 0.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 32.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	conditions_ssbtn->setStateImage(1,"conditions_button",
				  pp::Vec2d( 32.0/64.0, 0.0/64.0 ),
				  pp::Vec2d( 64.0/64.0, 32.0/64.0 ),
				  pp::Color::white );

	conditions_ssbtn->setStateImage(2,"conditions_button",
				  pp::Vec2d( 32.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 64.0/64.0, 64.0/64.0 ),
				  pp::Color::white );	

	conditions_ssbtn->setStateImage(3,"conditions_button",
				  pp::Vec2d( 0.0/64.0, 0.0/64.0 ),
				  pp::Vec2d( 32.0/64.0, 32.0/64.0 ),
				  pp::Color::white );	

	conditions_ssbtn->setState( (int)gameMgr->getCurrentRace().conditions );
	// wind
	wind_ssbtn = new pp::SSButton( pos,
				      pp::Vec2d(32, 32),
				      2 );
	wind_ssbtn->setStateImage(0,"wind_button",
				  pp::Vec2d( 0.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 32.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	wind_ssbtn->setStateImage(1,"wind_button",
				  pp::Vec2d( 32.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 64.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	wind_ssbtn->setState( (int)gameMgr->getCurrentRace().windy );

	// snow 
	snow_ssbtn = new pp::SSButton( pos,
				      pp::Vec2d(32, 32),
				      2 );
	snow_ssbtn->setStateImage(0,"snow_button",
				  pp::Vec2d( 0.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 32.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	snow_ssbtn->setStateImage(1,"snow_button",
				  pp::Vec2d( 32.0/64.0, 32.0/64.0 ),
				  pp::Vec2d( 64.0/64.0, 64.0/64.0 ),
				  pp::Color::white );

	snow_ssbtn->setState( (int)gameMgr->getCurrentRace().snowing );
	// XXX snow button doesn't do anything, so disable for now 
	snow_ssbtn->setEnabled(false);
	
    updateButtonEnabledStates();

    play_music( "start_screen" );
}

RaceSelect::~RaceSelect()
{
	delete back_btn;
	delete start_btn;
	delete race_listbox;
	delete conditions_ssbtn;
	delete snow_ssbtn;
	delete wind_ssbtn;
	delete mirror_ssbtn;
    delete desc_ta;		
}

void
RaceSelect::loop(double timeStep)
{
	update_audio();

    set_gl_options( GUI );

    clear_rendering_context();

    UIMgr.setupDisplay();

    if (getparam_ui_snow()) {
	update_ui_snow( timeStep, 
			(bool) ( wind_ssbtn != NULL && 
				  wind_ssbtn->getState() ) );
	draw_ui_snow();
    }

    theme.drawMenuDecorations();

    setWidgetPositionsAndDrawDecorations();

    UIMgr.draw();

    reshape( getparam_x_resolution(), getparam_y_resolution() );

    winsys_swap_buffers();
}

void
RaceSelect::setWidgetPositionsAndDrawDecorations()
{
    int w = getparam_x_resolution();
    int h = getparam_y_resolution();
    int box_width, box_height, box_max_y;
    int x_org, y_org;
    char *string;
    font_t font;
    char *current_course;
    int text_width, asc, desc;
    GLuint texobj;

    // set the dimensions of the box in which all widgets should fit
    box_width = 460;
    box_height = 310;
    box_max_y = h - 128;

    x_org = w/2 - box_width/2;
    y_org = h/2 - box_height/2;
    if ( y_org + box_height > box_max_y ) {
	y_org = box_max_y - box_height;
    }

    back_btn->setPosition(pp::Vec2d( x_org + 131 - back_btn->getWidth()/2.0,
		      42 ) );

    start_btn->setPosition(pp::Vec2d( x_org + 343 - start_btn->getWidth()/2.0,
		      42 ) );

	race_listbox->setPosition(pp::Vec2d( x_org,y_org + 221 ) );

	desc_ta->setPosition(pp::Vec2d( x_org,y_org + 66 ) );
    
	conditions_ssbtn->setPosition(
	    pp::Vec2d( x_org + box_width - 4*36 + 4,
			  y_org + 181 ) );

	wind_ssbtn->setPosition(
	    pp::Vec2d( x_org + box_width - 3*36 + 4 ,
			  y_org + 181 ) );

	snow_ssbtn->setPosition(
	    pp::Vec2d( x_org + box_width - 2*36 + 4,
			  y_org + 181 ) );

	mirror_ssbtn->setPosition(
	    pp::Vec2d( x_org + box_width - 1*36 + 4,
			  y_org + 181 ) );
    
    // Draw other stuff
    if ( !get_font_binding( "menu_label", font ) ) {
	print_warning( IMPORTANT_WARNING,
		       "Couldn't get font for binding menu_label" );
    } else {
	bind_font_texture( &font );
	string = "Select a race";
	get_font_metrics( &font, string, &text_width,  &asc, &desc );

	glPushMatrix();
	{
	    glTranslatef( x_org + box_width/2.0 - text_width/2.0,
			  y_org + 310 - asc, 
			  0 );

	    draw_string( &font, string );
	}
	glPopMatrix();
    }
    // Draw text indicating race requirements (if race not completed), 
    //   or results in race if race completed.
    drawStatusMsg( x_org, y_org );

    // Draw preview

	std::list<CourseData>::iterator elem;
	elem = race_listbox->getCurrentItem();
	current_course = (*elem).course;

    glDisable( GL_TEXTURE_2D );

    glColor4f( 1.0, 1.0, 1.0, 1.0 );
    glBegin( GL_QUADS );
    {
	glVertex2f( x_org+box_width-140, y_org+66 );
	glVertex2f( x_org+box_width, y_org+66 );
	glVertex2f( x_org+box_width, y_org+66+107 );
	glVertex2f( x_org+box_width-140, y_org+66+107 );
    }
    glEnd();

    glEnable( GL_TEXTURE_2D );

    if ( !get_texture_binding( current_course, &texobj ) ) {
	if ( !get_texture_binding( "no_preview", &texobj ) ) {
	    texobj = 0;
	}
    }

    glBindTexture( GL_TEXTURE_2D, texobj );

    glBegin( GL_QUADS );
    {
	glTexCoord2d( 0, 0);
	glVertex2f( x_org+box_width-136, y_org+70 );

	glTexCoord2d( 1, 0);
	glVertex2f( x_org+box_width-4, y_org+70 );

	glTexCoord2d( 1, 1);
	glVertex2f( x_org+box_width-4, y_org+70+99 );

	glTexCoord2d( 0, 1);
	glVertex2f( x_org+box_width-136, y_org+70+99 );
    }
    glEnd();
}


void
RaceSelect::updateRaceData()
{
  	gameMgr->getCurrentRace().course = (*curElem).course;
	gameMgr->getCurrentRace().name = (*curElem).name;
	gameMgr->getCurrentRace().description = (*curElem).description;
	gameMgr->getCurrentRace().mirrored = (bool) mirror_ssbtn->getState();
	gameMgr->getCurrentRace().conditions = (race_conditions_t) conditions_ssbtn->getState();
	gameMgr->getCurrentRace().windy = (bool) wind_ssbtn->getState();
	gameMgr->getCurrentRace().snowing = (bool) snow_ssbtn->getState();
}

void
RaceSelect::drawStatusMsg( int x_org, int y_org )
{
	
	PlayerCourseData data;
	Font font("race_requirements");
	Font labelfont("race_requirements_label");
	
	if(players[0].getOpenCourseData((*curElem).name,data)){
		
		char buff[BUFF_LEN];
		int minutes;
		int seconds;
		int hundredths;

		pp::Vec2d pos(x_org,y_org + 184);
		
		getTimeComponents( data.time, minutes, seconds, hundredths );

		labelfont.draw("Time:", pos);
		
		pos.x+=40;
		sprintf( buff, "%02d:%02d.%02d",minutes, seconds, hundredths);
		font.draw(buff, pos);
		
		pos.x+=80;
		labelfont.draw("Herring:", pos);
		
		pos.x+=55;
		sprintf( buff, "%03d", data.herring ); 
		font.draw(buff, pos);
		
		pos.x+=45;
		labelfont.draw("Score:", pos);

		pos.x+=42;
		sprintf( buff, "%06d", data.score );
		font.draw(buff, pos);
	}
		
	pp::Vec2d pos(x_org,y_org+50);
	labelfont.draw("Contributed by:",pos);
	pos.x+=106;	
	
	if((*curElem).contributed !=NULL){
		font.draw((*curElem).contributed,pos);
	}else{
		font.draw("Unknown",pos);
	}	
}


void
RaceSelect::updateButtonEnabledStates()
{
	start_btn->setEnabled( true );
}

void
RaceSelect::back()
{
	set_game_mode( GAME_TYPE_SELECT );
    UIMgr.setDirty();
}

void
RaceSelect::start()
{
    start_btn->setHighlight( true );
	updateRaceData();	
	loop( 0 );
    set_game_mode( LOADING );
}


void
RaceSelect::listboxItemChange()
{
	curElem=race_listbox->getCurrentItem();
	gameMgr->setCurrentRace(curElem);	
	updateRaceData();
	desc_ta->setText( (*curElem).description );
	updateButtonEnabledStates();
	UIMgr.setDirty();
}

bool
RaceSelect::keyPressEvent(SDLKey key)
{
	switch (key){
		case SDLK_UP:
		case SDLK_LEFT:
			race_listbox->gotoPrevItem();
	    	return true;
		case SDLK_RIGHT:
		case SDLK_DOWN:
			race_listbox->gotoNextItem();
	    	return true;
		case SDLK_RETURN: 
	    	start_btn->simulateMouseClick();
			UIMgr.setDirty();
			return true;
		case SDLK_ESCAPE: 
	    	back_btn->simulateMouseClick();
			UIMgr.setDirty();
			return true;
		case 'c': 
	    	conditions_ssbtn->simulateMouseClick();
	    	return true;
		case 'w': 
	    	wind_ssbtn->simulateMouseClick();
	    	return true;
		case 'm':
	    	mirror_ssbtn->simulateMouseClick();
	    	return true;
		default:
			return false;
	}	
}
